@@ -1,9 +1,11 @@
 /* GDA library
- * Copyright (C) 1998-2002 The GNOME Foundation.
+ * Copyright (C) 1998 - 2005 The GNOME Foundation.
  *
  * AUTHORS:
  *	Rodrigo Moya <rodrigo@gnome-db.org>
  *	Gonzalo Paniagua Javier <gonzalo@gnome-db.org>
+ *      Vivien Malerba <malerba@gnome-db.org>
+ *	Bas Driessen <bas.driessen@xobas.com>
  *
  * This Library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public License as
@@ -23,8 +25,9 @@
 
 #include <glib/ghash.h>
 #include <libgda/gda-data-model-hash.h>
+#include <libgda/gda-data-model-private.h>
 
-#define PARENT_TYPE GDA_TYPE_DATA_MODEL
+#define PARENT_TYPE GDA_TYPE_DATA_MODEL_BASE
 
 struct _GdaDataModelHashPrivate {
 	/* number of columns in each row */
@@ -32,6 +35,10 @@ struct _GdaDataModelHashPrivate {
 
 	/* the hash of rows, each item is a GdaRow */
 	GHashTable *rows;
+	gint number_of_hash_table_rows;
+
+	/* the mapping of a row to hash table entry */
+	GArray *row_map;
 };
 
 static void gda_data_model_hash_class_init (GdaDataModelHashClass *klass);
@@ -46,90 +53,148 @@ static GObjectClass *parent_class = NULL;
  */
 
 static gint
-gda_data_model_hash_get_n_rows (GdaDataModel *model)
+gda_data_model_hash_get_n_rows (GdaDataModelBase *model)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_HASH (model), -1);
-	return g_hash_table_size (GDA_DATA_MODEL_HASH (model)->priv->rows);
+
+	if (GDA_DATA_MODEL_HASH (model)->priv->row_map == NULL)
+		return -1;
+	else
+		return GDA_DATA_MODEL_HASH (model)->priv->row_map->len;
 }
 
 static gint
-gda_data_model_hash_get_n_columns (GdaDataModel *model)
+gda_data_model_hash_get_n_columns (GdaDataModelBase *model)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_HASH (model), -1);
 	return GDA_DATA_MODEL_HASH (model)->priv->number_of_columns;
 }
 
-static GdaFieldAttributes *
-gda_data_model_hash_describe_column (GdaDataModel *model, gint col)
+static GdaRow *
+gda_data_model_hash_get_row (GdaDataModelBase *model, gint row)
 {
+	gint hash_entry;
+
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_HASH (model), NULL);
 
-	return NULL;
+	/* get row according mapping */
+	hash_entry = g_array_index (GDA_DATA_MODEL_HASH (model)->priv->row_map, gint, row);
+
+	return (GdaRow *) g_hash_table_lookup (GDA_DATA_MODEL_HASH (model)->priv->rows,
+					       GINT_TO_POINTER (hash_entry));
+}
+
+static const GdaValue *
+gda_data_model_hash_get_value_at (GdaDataModelBase *model, gint col, gint row)
+{
+	const GdaRow *fields;
+
+	g_return_val_if_fail (GDA_IS_DATA_MODEL_HASH (model), NULL);
+
+	fields = gda_data_model_hash_get_row (model, row);
+	if (fields == NULL)
+		return NULL;
+
+	return (const GdaValue *) gda_row_get_value ((GdaRow *) fields, col);
 }
 
 static gboolean
-gda_data_model_hash_is_updatable (GdaDataModel *model)
+gda_data_model_hash_is_updatable (GdaDataModelBase *model)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_HASH (model), FALSE);
 	return TRUE;
 }
 
-static const GdaRow *
-gda_data_model_hash_append_row (GdaDataModel *model, const GList *values)
+static gboolean
+gda_data_model_hash_append_row (GdaDataModelBase *model, GdaRow *row)
 {
-	GdaRow *row;
-	gint cols;
+	gint cols, rownum_hash_new, rownum_array_new;
 
-	g_return_val_if_fail (GDA_IS_DATA_MODEL_HASH (model), NULL);
-	g_return_val_if_fail (values != NULL, NULL);
+	g_return_val_if_fail (GDA_IS_DATA_MODEL_HASH (model), FALSE);
+	g_return_val_if_fail (row != NULL, FALSE);
 
-	cols = g_list_length ((GList *) values);
+	cols = gda_row_get_length (row);
 	if (cols != GDA_DATA_MODEL_HASH (model)->priv->number_of_columns)
-		return NULL;
+		return FALSE;
 
-	/* create the GdaRow to add */
-	row = gda_row_new_from_list (model, values);
-	if (row) {
-		gda_data_model_hash_insert_row (
-			GDA_DATA_MODEL_HASH (model),
-			g_hash_table_size (GDA_DATA_MODEL_HASH (model)->priv->rows),
-			row);
-		gda_data_model_row_inserted (model, g_hash_table_size (GDA_DATA_MODEL_HASH (model)->priv->rows) - 1);
-		gda_data_model_changed (model);
-	}
+	/* get the new hash and array row number */
+	rownum_hash_new = GDA_DATA_MODEL_HASH (model)->priv->number_of_hash_table_rows;
+	rownum_array_new = GDA_DATA_MODEL_HASH (model)->priv->row_map->len;
 
-	return row;
+	gda_data_model_hash_insert_row (
+		GDA_DATA_MODEL_HASH (model),
+		rownum_hash_new,
+		row);
+
+	gda_row_set_number (row, rownum_array_new);
+
+	/* Append row number in row mapping array */
+	g_array_append_val (GDA_DATA_MODEL_HASH (model)->priv->row_map, rownum_hash_new);
+
+	/* emit update signals */
+	gda_data_model_row_inserted (GDA_DATA_MODEL (model), rownum_array_new);
+
+	/* increment the number of entries in the hash table */
+	GDA_DATA_MODEL_HASH (model)->priv->number_of_hash_table_rows++;
+
+	return TRUE;
 }
 
 static gboolean
-gda_data_model_hash_remove_row (GdaDataModel *model, const GdaRow *row)
+gda_data_model_hash_remove_row (GdaDataModelBase *model, GdaRow *row)
 {
+	gint i, cols, rownum;
+
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_HASH (model), FALSE);
-	return FALSE;
+	g_return_val_if_fail (row != NULL, FALSE);
+
+	cols = GDA_DATA_MODEL_HASH (model)->priv->number_of_columns;
+
+	for (i=0; i<cols; i++)
+		gda_row_set_value (row, i, NULL);
+
+	rownum = gda_row_get_number ((GdaRow *) row);
+
+	/* renumber following rows */
+	for (i=(rownum+1); i<GDA_DATA_MODEL_HASH (model)->priv->row_map->len; i++) 
+		gda_row_set_number ((GdaRow *) gda_data_model_get_row (GDA_DATA_MODEL (model), i), (i-1));
+
+	/* tag the row as being removed */	
+	gda_row_set_id ((GdaRow *) row, "R");
+	gda_row_set_number ((GdaRow *) row, -1);
+
+	/* remove entry from array */
+	g_array_remove_index (GDA_DATA_MODEL_HASH (model)->priv->row_map, rownum);
+
+	/* emit updated signals */
+	gda_data_model_row_removed (GDA_DATA_MODEL (model), gda_row_get_number ((GdaRow *) row));
+	
+	return TRUE;
 }
 
 static void
 free_hash (gpointer value)
 {
-	gda_row_free ((GdaRow *) value);
+	g_object_unref (value);
 }
 
 static void
 gda_data_model_hash_class_init (GdaDataModelHashClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-	GdaDataModelClass *model_class = GDA_DATA_MODEL_CLASS (klass);
+	GdaDataModelBaseClass *model_class = GDA_DATA_MODEL_BASE_CLASS (klass);
 
 	parent_class = g_type_class_peek_parent (klass);
 
 	object_class->finalize = gda_data_model_hash_finalize;
 	model_class->get_n_rows = gda_data_model_hash_get_n_rows;
 	model_class->get_n_columns = gda_data_model_hash_get_n_columns;
-	model_class->describe_column = gda_data_model_hash_describe_column;
 	model_class->get_row = gda_data_model_hash_get_row;
 	model_class->get_value_at = gda_data_model_hash_get_value_at;
 	model_class->is_updatable = gda_data_model_hash_is_updatable;
+	model_class->append_values = NULL;
 	model_class->append_row = gda_data_model_hash_append_row;
+	model_class->update_row = NULL;
 	model_class->remove_row = gda_data_model_hash_remove_row;
 }
 
@@ -141,7 +206,9 @@ gda_data_model_hash_init (GdaDataModelHash *model, GdaDataModelHashClass *klass)
 	/* allocate internal structure */
 	model->priv = g_new0 (GdaDataModelHashPrivate, 1);
 	model->priv->number_of_columns = 0;
+	model->priv->number_of_hash_table_rows = 0;
 	model->priv->rows = NULL;
+	model->priv->row_map = NULL;
 }
 
 static void
@@ -155,6 +222,8 @@ gda_data_model_hash_finalize (GObject *object)
 	g_hash_table_destroy (model->priv->rows);
 	model->priv->rows = NULL;
 
+	g_array_free (model->priv->row_map, TRUE);
+
 	g_free (model->priv);
 	model->priv = NULL;
 
@@ -211,29 +280,6 @@ gda_data_model_hash_new (gint cols)
 	return model;
 }
 
-/**
- * gda_data_model_hash_get_value_at
- * @model: the #GdaDataModelHash to retrieve the value from.
- * @col: column number (starting from 0).
- * @row: row number (starting from 0).
- *
- * Retrieves the value at a specified column and row.
- * Returns: a pointer to a #GdaValue.
- */
-const GdaValue *
-gda_data_model_hash_get_value_at (GdaDataModel *model, gint col, gint row)
-{
-	const GdaRow *fields;
-
-	g_return_val_if_fail (GDA_IS_DATA_MODEL_HASH (model), NULL);
-
-	fields = gda_data_model_hash_get_row (model, row);
-	if (fields == NULL)
-		return NULL;
-
-	return (const GdaValue *) gda_row_get_value ((GdaRow *) fields, col);
-}
-
 /**
  * gda_data_model_hash_insert_row
  * @model: the #GdaDataModelHash which is gonna hold the row.
@@ -263,26 +309,8 @@ gda_data_model_hash_insert_row (GdaDataModelHash *model,
 
 	g_hash_table_insert (model->priv->rows,
 			     GINT_TO_POINTER (rownum), row);
+	g_object_ref (row);
 	gda_data_model_row_inserted (GDA_DATA_MODEL (model), rownum);
-	gda_data_model_changed (GDA_DATA_MODEL (model));
-}
-
-/**
- * gda_data_model_hash_get_row
- * @model: the GdaDataModelHash
- * @row: row number
- *
- * Retrieves a row from the underlying hash table.
- *
- * Returns: a #GdaRow or NULL if the requested row is not in the hash table.
- */
-const GdaRow *
-gda_data_model_hash_get_row (GdaDataModel *model, gint row)
-{
-	g_return_val_if_fail (GDA_IS_DATA_MODEL_HASH (model), NULL);
-
-	return (const GdaRow *) g_hash_table_lookup (GDA_DATA_MODEL_HASH (model)->priv->rows,
-						     GINT_TO_POINTER (row));
 }
 
 /**
@@ -315,6 +343,8 @@ gda_data_model_hash_set_n_columns (GdaDataModelHash *model, gint cols)
 void
 gda_data_model_hash_clear (GdaDataModelHash *model)
 {
+	gint i;
+
 	g_return_if_fail (GDA_IS_DATA_MODEL_HASH (model));
 
 	if (model->priv->rows != NULL)
@@ -322,5 +352,23 @@ gda_data_model_hash_clear (GdaDataModelHash *model)
 	model->priv->rows = g_hash_table_new_full (g_direct_hash,
 						   g_direct_equal,
 						   NULL, free_hash);
+
+	/* free array if exists */
+	if (model->priv->row_map != NULL)
+	{
+		g_array_free (model->priv->row_map, TRUE);
+		model->priv->row_map = NULL;
+	}
+
+	/* get number of entries in data model */
+	model->priv->number_of_hash_table_rows = gda_data_model_get_n_rows (GDA_DATA_MODEL (model));
+
+	/* create row mapping array */
+	model->priv->row_map = g_array_new (FALSE, FALSE, sizeof (gint));
+
+        /* Initialize row mapping array */
+        for (i = 0; i < model->priv->number_of_hash_table_rows; i++)
+                g_array_append_val (model->priv->row_map, i);
+
 }
 
