@@ -1,5 +1,5 @@
 /* GDA common library
- * Copyright (C) 1999-2001 The Free Software Foundation
+ * Copyright (C) 1998-2002 The GNOME Foundation.
  *
  * AUTHORS:
  *	Rodrigo Moya <rodrigo@gnome-db.org>
@@ -23,16 +23,37 @@
 #if !defined(__gda_util_h__)
 #  define __gda_util_h__
 
-#include <glib.h>
+#include <glib/ghash.h>
+#include <glib/glist.h>
+#include "gda-parameter.h"
+#include "gda-row.h"
+#include "gda-connection.h"
 
 G_BEGIN_DECLS
 
+const gchar *gda_type_to_string (GdaValueType type);
+GdaValueType gda_type_from_string (const gchar *str);
+
+GList       *gda_string_hash_to_list (GHashTable *hash_table);
+
+/*
+ * SQL parsing utilities
+ */
+
+gchar *gda_sql_replace_placeholders (const gchar *sql, GdaParameterList *params);
+
 /*
  * File management utility functions
  */
 
-gchar    *gda_file_load (const gchar *uri);
-gboolean  gda_file_save (const gchar *uri, const gchar *buffer, gint len);
+gchar    *gda_file_load (const gchar *filename);
+gboolean  gda_file_save (const gchar *filename, const gchar *buffer, gint len);
+
+/*
+ * Help to implement providers
+ */
+gint      gda_provider_get_schema_nb_columns (GdaConnectionSchema schema);
+gboolean  gda_provider_init_schema_model     (GdaDataModel *model, GdaConnectionSchema schema);
 
 G_END_DECLS
 
